///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.ColorReducers;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.ColorReducers
{
	public class LnAbsXPlusSAsymptotic : AsymptoticColorReducer<DummyScriptConfiguration>
	{
		public override string Description => "x / (s2 + x); x = |(s1 + |rs|).LnA|";

		public override void Validate( IFluonConfiguration fc, IColorReducerConfiguration crp )
		{
			base.Validate( fc, crp );

			if( crp.S2 < 1 )
				throw new ArgumentException( $"{crp.Name}: crp.S2 < 1" );
		}

		protected override double ReduceAsymptoticallyToUnitary( IFluonContext fx, IFluonConfiguration fc, IColorReducerConfiguration crp
			, double x )
		{
			x = (crp.S1 + x.Abs()).LnA().Abs();

			return x / (crp.S2 + x);
		}
	}
}
