///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.ColorReducers;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.ColorReducers
{
	public class ScaleUnitaryDoubleCurve : NonLinearOnUnitaryColorReducer<DummyScriptConfiguration>
	{
		public override string Description => "rs.ScaleUnitaryDoubleCurve( s1, s2 )";

		public override void Validate( IFluonConfiguration fc, IColorReducerConfiguration crp )
		{
			base.Validate( fc, crp );

			if( crp.S1 <= -1 || crp.S1 >= 1 )
				throw new ArgumentException( $"{crp.Name}: crp.S1 <= -1 || crp.S1 >= 1" );
			if( crp.S2 <= -1 || crp.S2 >= 1 )
				throw new ArgumentException( $"{crp.Name}: crp.S2 <= -1 || crp.S2 >= 1" );
		}

		protected override double ApplyNonLinearUnitary( IFluonContext fx, IFluonConfiguration fc, IColorReducerConfiguration crp, double x )
		{
			return x.ScaleUnitaryDoubleCurve( crp.S1, crp.S2 );
		}
	}
}
