///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;
using Fluogen.Dependz.MdnReducers;
using Got.Libraries;

namespace Fluogen.DeployedScripts.MdnReducers
{
	public class WiringOfReality : MdnReducerWithAccumulatorThatAvoidsEvEvpConvergence<DummyScriptConfiguration>
	{
		public override string Description => "Accumulate: a + 1 - (ev - evp).LoopLnA( s1 ) / et.LoopLnA( s1 - 1 )";

		private double EtLoopLn;

		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			if( fc.MdnReducerS1 <= 0 || !fc.MdnReducerS1.IsInteger() )
				throw new ArgumentException( $"{nameof( WiringOfReality )}: MdnReducerS1 <= 0 || !IsInteger( MdnReducerS1 )" );
		}

		public override void InitializeGlobalState( IFluonConfiguration fc )
		{
			double et = fc.EscapeThreshold;
			int count = (int)fc.MdnReducerS1;

			EtLoopLn = et.LoopLnA( count - 1 );
		}

		public override ColorDeterminant Accumulate( IFluonContext fx, IFluonConfiguration fc, IPixelContext px, Mdn x, Mdn y, Mdn yp
			, double ev, double evp, ColorDeterminant a, int ri )
		{
			int count = (int)fc.MdnReducerS1;

			return a.Add( 1 - (ev - evp).LoopLnA( count ) / EtLoopLn );
		}
	}
}
