///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Fluogen.DeployedScripts.Libraries;

// #lib PaletteToneExtensions

namespace Fluogen.DeployedScripts.Palettes
{
	public class Beige : BitonalGradientPalette<DummyScriptConfiguration>
	{
		protected override IBitonalGradientPaletteConfiguration GetSpecificConfiguration( IFluonConfiguration fc
			, IPaletteContext paletteContext )
		{
			var configuration = base.GetSpecificConfiguration( fc, paletteContext );

			var colorModel = paletteContext.OC.ColorModel.Value;

			if( PX.OC.PaletteVariant != PaletteVariant.Specific )
			{
				configuration.CaramelTone1( colorModel );
				configuration.CaramelTone2( colorModel );
			}

			return configuration;
		}

		protected override IGradientPaletteMarkList GetSpecificVariantMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			int hue = PaletteToneExtensions.CaramelHueHsl;
			int edgesS = BGPC.OC.EdgesSaturation.GetValueOrDefault( 10000 );
			int edgesL = BGPC.OC.EdgesLightness.GetValueOrDefault( 9500 );
			int tintS = BGPC.OC.TintSaturation.GetValueOrDefault( 10000 );
			int tintL = BGPC.OC.TintLightness.GetValueOrDefault( 10000 );

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new BitonalGradientPaletteVariant( fc, PX, BGPC
						, new BitonalGradientPaletteToneMarkDatas
							{
								{ 0.0000, hue, edgesS, edgesL },
								{ 0.1000, hue,  10000,   9000 },
								{ 0.2000, hue,  10000,   9600 },
								{ 0.3000, hue,  10000,   7200 },
								{ 0.4000, hue,  10000,   8000 },
								{ 0.5000, hue,  tintS,  tintL, true },
								{ 0.6000, hue,  10000,   8000 },
								{ 0.7000, hue,  10000,   7200 },
								{ 0.8000, hue,  10000,   9600 },
								{ 0.9000, hue,  10000,   9000 },
								{ 1.0000, hue, edgesS, edgesL }
							}
						).ToMarkList();
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
