///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.Palettes
{
	public class BlackWhite : Palette<DummyScriptConfiguration>
	{
		public override int FullSize => 2 * UInt16.MaxValue;
		public override int QuickSize => 256;

		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			var poc = fc.PaletteOverridingConfiguration;

			if( poc.PaletteS1 != null && (poc.PaletteS1 < -1 || poc.PaletteS1 > 1) )
				throw new ArgumentException( $"{nameof( BlackWhite )}: PaletteS1 < -1 || PaletteS1 > 1" );

			if( poc.PaletteS2 != null && (poc.PaletteS2 < -1 || poc.PaletteS2 > 1) )
				throw new ArgumentException( $"{nameof( BlackWhite )}: PaletteS2 < -1 || PaletteS2 > 1" );

			if( poc.PaletteS3 != null && (poc.PaletteS3 < -1 || poc.PaletteS3 > 1) )
				throw new ArgumentException( $"{nameof( BlackWhite )}: PaletteS3 < -1 || PaletteS3 > 1" );
		}

		protected override void InitializeCache( IFluonConfiguration fc )
		{
		}

		protected override IList<PixelRgbColor> CacheColors()
		{
			var colors = new PixelRgbColor[ ActualSize ];

			var poc = PX.OverriddenConfiguration;

			double redScale = poc.PaletteS1.Value;
			double greenScale = poc.PaletteS2.Value;
			double blueScale = poc.PaletteS3.Value;

			int halfSize_1 = ActualSize / 2 - 1;

			for( int i = 0; i <= halfSize_1; i++ )
			{
				double s = (double)i / halfSize_1;

				double red = (redScale >= 0 ? 0 : 1) + s * redScale;
				double green = (greenScale >= 0 ? 0 : 1) + s * greenScale;
				double blue = (blueScale >= 0 ? 0 : 1) + s * blueScale;

				var color = PixelRgbColor.FromRgbScalar( red, green, blue );

				colors[ i ] = color;
				colors[ ActualSize - 1 - i ] = color;
			}

			return colors;
		}

		protected override PixelRgbColor ComputeColor( IFluonConfiguration fc, double scalar )
		{
			throw new InvalidOperationException();
		}
	}
}
