///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Fluogen.DeployedScripts.Libraries;

// #lib PaletteToneExtensions

namespace Fluogen.DeployedScripts.Palettes
{
	public class Brown : BitonalGradientPalette<DummyScriptConfiguration>
	{
		protected override IBitonalGradientPaletteConfiguration GetSpecificConfiguration( IFluonConfiguration fc
			, IPaletteContext paletteContext )
		{
			var configuration = base.GetSpecificConfiguration( fc, paletteContext );

			var colorModel = paletteContext.OC.ColorModel.Value;

			if( PX.OC.PaletteVariant != PaletteVariant.Specific )
			{
				configuration.PassionTone1( colorModel );

				configuration.MakeMonotonal();
			}

			return configuration;
		}

		protected override IGradientPaletteMarkList GetSpecificVariantMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			int hue = PaletteToneExtensions.PassionHueHsl;
			int edgesS = BGPC.OC.EdgesSaturation.GetValueOrDefault( 3500 );
			int edgesL = BGPC.OC.EdgesLightness.GetValueOrDefault( 500 );
			int tintS = BGPC.OC.TintSaturation.GetValueOrDefault( 2500 );
			int tintL = BGPC.OC.TintLightness.GetValueOrDefault( 10000 );

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new BitonalGradientPaletteVariant( fc, PX, BGPC
						, new BitonalGradientPaletteToneMarkDatas
							{
								{ 0.0000, hue, edgesS, edgesL },
								{ 0.1000, hue,   3500,   1500 },
								{ 0.3500, hue,   3000,   6000 },
								{ 0.5000, hue,  tintS,  tintL, true },
								{ 0.6500, hue,   3000,   6000 },
								{ 0.9000, hue,   3500,   1500 },
								{ 1.0000, hue, edgesS, edgesL }
							}
						).ToMarkList();
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
