///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.Palettes
{
	/// <summary>
	/// Inspired by https://web.mit.edu/jorloff/www/chaosTalk/mandelbrot/davideck-js/MB-info.html
	/// </summary>
	public class EarthAndSky : MonotoneCubicGradientPalette<DummyScriptConfiguration>
	{
		protected int Hue1( ColorModelName colorModel ) => colorModel.SelectForColorModel( 1200, 1800 );
		protected int Hue2( ColorModelName colorModel ) => colorModel.SelectForColorModel( 0, 700 );
		protected int Hue3( ColorModelName colorModel ) => colorModel.SelectForColorModel( 7400, 8200 );

		protected override IGradientPaletteMarkList GetMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new GradientPaletteMarkList
					{
						Hsl( 0.00,  1200,     0, 10000 ),
						Hsl( 0.15,  1200, 10000,  6000 ),
						Hsl( 0.35,     0, 10000,  4000 ),
						Hsl( 0.65,  7400, 10000,  5000, true ),
						Hsl( 0.80,  7400, 10000,  7000 ),
						Hsl( 0.90,  7400, 10000,  8000 ),
						Hsl( 1.00,  7400,     0, 10000 )
					};
				case ColorModelName.Hcj:
					return new GradientPaletteMarkList
					{
						Hcj( 0.00, Hue1( colorModel ),     0, 10000 ),
						Hcj( 0.15, Hue1( colorModel ), 10000,  7000 ),
						Hcj( 0.35, Hue2( colorModel ), 10000,  3000 ),
						Hcj( 0.65, Hue3( colorModel ), 10000,  2900, true ),
						Hcj( 0.80, Hue3( colorModel ), 10000,  4800 ),
						Hcj( 0.90, Hue3( colorModel ), 10000,  6300 ),
						Hcj( 1.00, Hue3( colorModel ),     0, 10000 )
					};
				//case ColorModelName.Hcl:
				//	return new GradientPaletteMarkList
				//	{
				//  PaletteExtensions.GetMarkFromTypeableHsl( 0.00, Hue1( colorModel ), 10000, 10000, colorModel ),
				//  PaletteExtensions.GetMarkFromTypeableHsl( 0.15, Hue1( colorModel ), 10000,  8300, colorModel ),
				//  PaletteExtensions.GetMarkFromTypeableHsl( 0.35, Hue2( colorModel ), 10000,  4000, colorModel ),
				//  PaletteExtensions.GetMarkFromTypeableHsl( 0.65, Hue3( colorModel ), 10000,  1500, colorModel ),
				//  PaletteExtensions.GetMarkFromTypeableHsl( 0.80, Hue3( colorModel ), 10000,  5000, colorModel ),
				//  PaletteExtensions.GetMarkFromTypeableHsl( 0.90, Hue3( colorModel ), 10000,  7000, colorModel ),
				//  PaletteExtensions.GetMarkFromTypeableHsl( 1.00, Hue3( colorModel ), 10000, 10000, colorModel )
				//	};//?
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
