///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using ColorManagement.Conversions;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Fluogen.DeployedScripts.Libraries;
using Got.Libraries;

// #lib PaletteToneExtensions

namespace Fluogen.DeployedScripts.Palettes
{
	public class ForestDream : BitonalGradientPalette<DummyScriptConfiguration>
	{
		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			var poc = fc.PaletteOverridingConfiguration;

			if( poc.PaletteS1 != null && (poc.PaletteS1 < 0 || poc.PaletteS1 > TypeableColorConverter.IntegerMax
				|| !poc.PaletteS1.Value.IsInteger()) )
			{
				throw new ArgumentException( $"{nameof( ForestDream )}: PaletteS1 < 0 || PaletteS1 > {TypeableColorConverter.IntegerMax}"
					+ $"|| !IsInteger( PaletteS1 )" );
			}
		}

		protected override IPaletteConfiguration GetPaletteConfiguration( IFluonConfiguration fc )
		{
			var configuration = base.GetPaletteConfiguration( fc );

			configuration.PaletteS1 = 7700;

			return configuration;
		}

		protected override IBitonalGradientPaletteConfiguration GetSpecificConfiguration( IFluonConfiguration fc
			, IPaletteContext paletteContext )
		{
			var configuration = base.GetSpecificConfiguration( fc, paletteContext );

			var colorModel = paletteContext.OC.ColorModel.Value;

			if( PX.OC.PaletteVariant != PaletteVariant.Specific )
			{
				configuration.ForestTone1( colorModel );
				configuration.LavenderTone2( colorModel );
			}

			return configuration;
		}

		protected override IGradientPaletteMarkList GetSpecificVariantMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			int hue1 = PaletteToneExtensions.ForestHueHsl;
			int hue2 = PaletteToneExtensions.LavenderHueHsl;
			int edgesS = BGPC.OC.EdgesSaturation.GetValueOrDefault( 10000 );
			int edgesL = BGPC.OC.EdgesLightness.GetValueOrDefault( 0 );
			int tintS = BGPC.OC.TintSaturation.GetValueOrDefault( 10000 );
			int tintL = BGPC.OC.TintLightness.GetValueOrDefault( 9800 );
			int warmth = (int)PX.OC.PaletteS1.Value;

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new BitonalGradientPaletteVariant( fc, PX, BGPC
						, new BitonalGradientPaletteToneMarkDatas
							{
								{ 0.0000, hue1, edgesS, edgesL },
								{ 0.1000, hue1,  10000,    650 },
								{ 0.2167, hue1,  10000,   3050 },
								{ 0.3333, hue1,  10000,   7600 },
								{ 0.4900, hue1,  tintS,  tintL, true },
								{ 0.6000, hue1,  10000,   5200 },
								{ 0.7500, hue2,  10000,   7100 },
								{ 0.8167, hue2, warmth,   5200 },
								{ 0.8833, hue2,  10000,   3200 },
								{ 0.9500, hue2,  10000,   1600 },
								{ 1.0000, hue2, edgesS, edgesL }
							}
						).ToMarkList();
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
