///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.Palettes
{
	public class MandelbrotCauliflowerSpiral : MonotoneCubicGradientPalette<DummyScriptConfiguration>
	{
		protected override IPaletteConfiguration GetPaletteConfiguration( IFluonConfiguration fc )
		{
			var configuration = base.GetPaletteConfiguration( fc );

			configuration.ReducedColorKaleidoscope = ReducedColorKaleidoscopeMode.ReverseAll;

			return configuration;
		}

		protected override IGradientPaletteMarkList GetMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new GradientPaletteMarkList
					{
						Hsl( 0.0000,   950, 10000,     0 ),
						Hsl( 0.0500,   960, 10000,  1600 ),
						Hsl( 0.1167,   970, 10000,  3200 ),
						Hsl( 0.1833,   990,  8000,  5200 ),
						Hsl( 0.2500,  1000, 10000,  7600 ),
						Hsl( 0.3280,   700, 10000,  2900 ),
						Hsl( 0.4000,  1150, 10000, 10000, true ),
						Hsl( 0.4810,  6000, 10000,  9100 ),
						Hsl( 0.5650,  6000, 10000,  2800 ),
						Hsl( 0.6667,  6000, 10000,  7600 ),
						Hsl( 0.7833,  6000, 10000,  3050 ),
						Hsl( 0.9000,  6000, 10000,   650 ),
						Hsl( 1.0000,  6000, 10000,     0 )
					};
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
