///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using ColorManagement.Conversions;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Fluogen.DeployedScripts.Libraries;
using Got.Libraries;

// #lib PaletteToneExtensions

namespace Fluogen.DeployedScripts.Palettes
{
	public class MyDarkness : BitonalGradientPalette<DummyScriptConfiguration>
	{
		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			var poc = fc.PaletteOverridingConfiguration;

			if( poc.PaletteS1 != null && (poc.PaletteS1 < 0 || poc.PaletteS1 > TypeableColorConverter.IntegerMax
				|| !poc.PaletteS1.Value.IsInteger()) )
			{
				throw new ArgumentException( $"{nameof( MyDarkness )}: PaletteS1 < 0 || PaletteS1 > {TypeableColorConverter.IntegerMax}"
					+ $"|| !IsInteger( PaletteS1 )" );
			}
		}

		protected override IPaletteConfiguration GetPaletteConfiguration( IFluonConfiguration fc )
		{
			var configuration = base.GetPaletteConfiguration( fc );

			configuration.PaletteS1 = TypeableColorConverter.IntegerMax;

			return configuration;
		}

		protected override IBitonalGradientPaletteConfiguration GetSpecificConfiguration( IFluonConfiguration fc
			, IPaletteContext paletteContext )
		{
			var configuration = base.GetSpecificConfiguration( fc, paletteContext );

			var colorModel = paletteContext.OC.ColorModel.Value;

			if( PX.OC.PaletteVariant != PaletteVariant.Specific )
			{
				configuration.TangerineTone1( colorModel );

				configuration.MakeMonotonal();
			}

			return configuration;
		}

		protected override IGradientPaletteMarkList GetSpecificVariantMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			int hue = PaletteToneExtensions.TangerineHueHsl;
			int edgesS = BGPC.OC.EdgesSaturation.GetValueOrDefault( 10000 );
			int edgesL = BGPC.OC.EdgesLightness.GetValueOrDefault( 0 );
			int tintS = BGPC.OC.TintSaturation.GetValueOrDefault( 10000 );
			int tintL = BGPC.OC.TintLightness.GetValueOrDefault( 5000 );
			int warmth = (int)PX.OC.PaletteS1.Value;

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new BitonalGradientPaletteVariant( fc, PX, BGPC
						, new BitonalGradientPaletteToneMarkDatas
							{
								{ 0.0000, hue, edgesS, edgesL },
								{ 0.3700, hue,  10000,   7000 },
								{ 0.5000, hue,  tintS,  tintL, true },
								{ 0.6000, hue, warmth,   7000 },
								{ 1.0000, hue, edgesS, edgesL }
							}
						).ToMarkList();
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
