///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using ColorManagement.ColorModels;
using ColorManagement.Conversions;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.Palettes
{
	public class TestHcjSolidHue : Palette<DummyScriptConfiguration>
	{
		public override int FullSize => 1 << 16;
		public override int QuickSize => 256;

		protected int H { get; } = 2200;
		protected int C { get; } = 5000;
		protected int J { get; } = 500; // Max 865?

		protected override void InitializeCache( IFluonConfiguration fc )
		{
		}

		protected override PixelRgbColor ComputeColor( IFluonConfiguration fc, double scalar )
		{
			int hue1 = fc.BitonalGradientPaletteOverridingConfiguration.ToneHue1 != null
				? fc.BitonalGradientPaletteOverridingConfiguration.ToneHue1.Value : H;
			int hue2 = fc.BitonalGradientPaletteOverridingConfiguration.ToneHue2 != null
				? fc.BitonalGradientPaletteOverridingConfiguration.ToneHue2.Value : H;
			int c1 = fc.BitonalGradientPaletteOverridingConfiguration.ToneSaturation1 != null
				? fc.BitonalGradientPaletteOverridingConfiguration.ToneSaturation1.Value : C;

			double max = TypeableColorConverter.IntegerMax;
			double h = hue1 / max;
			double c = c1 / max;
			double j = J / max;

			IHcj hcj = new Hcj( h, c, j );

			var rgb = HcjConverter.ToRgb( hcj, fc.HclColorSpace, fc.BringHclInGamutOfColorSpace );

			var rgb16Bits = Rgb16Bits.FromRgbScalar( rgb );

			return PixelRgbColor.FromRgb16Bits( rgb16Bits.R, rgb16Bits.G, rgb16Bits.B );
		}
	}
}
